/*=============================================================================
	UnXC_Game.h: Unreal game class.
=============================================================================*/

/*-----------------------------------------------------------------------------
	Unreal game engine.
-----------------------------------------------------------------------------*/
#if _MSC_VER
#pragma pack (push,4)
#endif

#ifndef XC_ENGINE_API
#define XC_ENGINE_API DLL_IMPORT
#endif

#ifndef NAMES_ONLY
#define AUTOGENERATE_NAME(name) extern XC_ENGINE_API FName XC_ENGINE_##name;
#define AUTOGENERATE_FUNCTION(cls,idx,name)
#endif

AUTOGENERATE_NAME(XC_Engine)
AUTOGENERATE_NAME(XC_GameEngine)
AUTOGENERATE_NAME(XC_ConnectionHandler)

#ifndef NAMES_ONLY

//Do remove debugging instructions
//#define CACUS_DEBUG

#include "XC_CoreObj.h"

//These variables should persist, so we can't really afford to let the garbage collector to mess this up

#ifndef DISABLE_COLLISION
	XC_ENGINE_API extern FCollisionCacus*	CollisionCacus;
#endif

/*
static void TestInverse()
{
	for ( FLOAT f=0 ; f<33000 ; f = f*1.4f + 0.005f )
	{
		FLOAT inv = _Reciprocal(f);
		FLOAT div = 1.f/f;
		debugf( NAME_Log, TEXT("Test inverse of %f: %f, %f, %f"), f, div, inv, div-inv);
	}
}
*/

#ifndef DISABLE_SCRIPT
class AXC_DummyPawn : private APawn
{
	friend class UXC_GameEngine;
	
	public:
	DECLARE_FUNCTION(NewMapName);			//539 - hack
	DECLARE_FUNCTION(NewPlayerCanSeeMe);	//532 - hack
	DECLARE_FUNCTION(NewDynArrayElement);	//10 - version 10
	DECLARE_FUNCTION(NewDynArrayLength);	//640 - version 10
	DECLARE_FUNCTION(NewDynArrayInsert);	//641 - version 10
	DECLARE_FUNCTION(NewDynArrayRemove);	//642 - version 10
	DECLARE_FUNCTION(execAddToPackagesMap);	//1718 - version 11
    DECLARE_FUNCTION(PreLoginNative);		//3550 - internal
    DECLARE_FUNCTION(AdminLoginNative);		//3551 - internal
    DECLARE_FUNCTION(execCollidingActors);	//3552 - version 12
    DECLARE_FUNCTION(execIsInPackageMap);	//1719 - version 13
	//TODO:
	//native(3554) static final function iterator ConnectedDests( NavigationPoint Start, out Actor End, out int ReachSpecIdx, out int PathArrayIdx);
	/*
	native(1715) final function Actor SpawnAct
	(
		Class<Actor>	SpawnClass,
		vector		SpawnLocation,
		optional rotator	SpawnRotation,
		optional name	ActorName,
		optional Actor	SpawnOwner,
		optional Pawn	SpawnInstigator,
		optional Actor	Template,
		optional bool	bMayColFail
	);
	*/
//    DECLARE_CLASS(APawn,AActor,0|CLASS_Abstract,XC_Engine)
};
#endif

//
// Editor entry point
//
#ifndef KILL_ENGINE
class XC_ENGINE_API UXC_EditorLoader : public UObject
{
	public:
	DECLARE_CLASS(UXC_EditorLoader,UObject,0,XC_Engine)
};


//
// The Unreal game engine, subclassed.
//

#define XCGE_Defaults GetDefault<UXC_GameEngine>()
class XC_ENGINE_API UXC_GameEngine : public UGameEngine
{
	public:
	
	INT				DummyData_0;	//Safety for 451 games

//	ULevel*			GLevel2; 
	INT				Version;

	//Config
	TArray<FString>	NoBrushTrackerFix;
	BITFIELD		bDisableTimingFix:1 GCC_PACK(4);
	BITFIELD		bDisableBrushTracker:1;
	BITFIELD		bSortMaplistByFolder:1;
	BITFIELD		bSortMaplistGlobal:1;
	BITFIELD		bAutoTravelManager:1;
	BITFIELD		bCacheConvertAtJoin:1;
	BITFIELD		bCollisionHashHook:1;
	BITFIELD		bFasterUpload:1;
	BITFIELD		bEnableDebugLogs:1;
	BITFIELD		bUseNewRelevancy:1;
	BITFIELD		bUseSetEnemyHook:1;
	BITFIELD		bUseLevelHook:1;

	INT				DummyData_1 GCC_PACK(4);

	//Hook data
	AActor*			PreLoginHooks[12];
	AActor*			AdminLoginHook;

	INT				LastMapIdx;
	FStringNoInit	MapCachedPrefix;
	TArray<FString>	MapCache;
	
	FGenericSystemDispatcher XCGESystems;

	//Hook status
	BITFIELD		bHackingTracker:1 GCC_PACK(4);
	BITFIELD		bHackedEngine:1; //Main switch for general engine hacks (GetWeapon, PreLogin)
	BITFIELD		bSetEnemy:1;
	BITFIELD		bRocketTick:1;
	BITFIELD		bSetOrders:1;
	BITFIELD		bCanEditPackageMap:1; //It's a XCGE status var, it qualifies
	BITFIELD		b451Setup:1;
	BITFIELD		b440Net:1;
	BITFIELD		bHTTP_DLMGR:1;


	INT				DummyData_2 GCC_PACK(4); //Safety for 451 games

	// Constructors.
	UXC_GameEngine();
#ifndef DISABLE_STATICS
	void StaticConstructor();
#endif
	void Destroy();
	void Serialize( FArchive& Ar );
	
	//UEngine interface
	void Init();
	void Exit();
	void Tick( FLOAT DeltaSeconds );
	UBOOL Exec( const TCHAR* Cmd, FOutputDevice& Ar=*GLog );

	virtual UBOOL Browse( FURL URL, const TMap<FString,FString>* TravelInfo, FString& Error );
	virtual ULevel* LoadMap( const FURL& URL, UPendingLevel* Pending, const TMap<FString,FString>* TravelInfo, FString& Error );
	virtual void NotifyLevelChange();

//	virtual ULevel* CreateLevel( const TCHAR* MapName, FString& Error );
#ifndef DISABLE_SCRIPT
	void ReplaceGetWeaponFunc( UBOOL bSet);
	void ReplacePreLoginFunc( UBOOL bSet);
	void ReplaceSetEnemyFunc( UBOOL bSet);
	void ReplaceRocketTickFunc( UBOOL bSet);
	void ReplaceSetOrdersFunc( UBOOL bSet);
	void ReplaceShowPathFunc( UBOOL bSet);
	void ReplaceShowInventoryFunc( UBOOL bSet);
	void ReplaceAdminLoginFunc( UBOOL bSet);
#endif

//	void CompileText( FString FileName);
	void HookEngine( ULevel* aLevel );
	void UnHookEngine();
	void HookNatives( UBOOL bSet);
	void CleanupTransientRefs();
#ifndef DISABLE_COLLISION
	void SetupCollisionCacus( ULevel* Lev);
	void DeleteCollisionCacus( ULevel* Lev);
#endif

	//Exec function
	void PerformSetCommand( const TCHAR* Str, FOutputDevice& Ar, UBOOL bNotifyObjectOfChange );
	void ProcessPreLoginHooks(const FString& Options, const FString& Address, FString& Error, FString& FailCode);

	DECLARE_CLASS(UXC_GameEngine,UGameEngine,CLASS_Config|CLASS_Transient,XC_Engine)

/*
	// Variables.
	ULevel*			GLevel;
	ULevel*			GEntry;
	UPendingLevel*	GPendingLevel;
	FURL			LastURL;
	TArray<FString> ServerActors;
	TArray<FString> ServerPackages;


	// UObject interface.
	void Serialize( FArchive& Ar );
	void Destroy();

	// UEngine interface.
	void Init();
	void Exit();
	void Tick( FLOAT DeltaSeconds );
	void Draw( UViewport* Viewport, UBOOL Blit=1, BYTE* HitData=NULL, INT* HitSize=NULL );
	UBOOL Exec( const TCHAR* Cmd, FOutputDevice& Ar=*GLog );
	void MouseDelta( UViewport*, DWORD, FLOAT, FLOAT );
	void MousePosition( class UViewport*, DWORD, FLOAT, FLOAT );
	void Click( UViewport*, DWORD, FLOAT, FLOAT );
	void SetClientTravel( UPlayer* Viewport, const TCHAR* NextURL, UBOOL bItems, ETravelType TravelType );
	FLOAT GetMaxTickRate();
	INT ChallengeResponse( INT Challenge );
	void SetProgress( const TCHAR* Str1, const TCHAR* Str2, FLOAT Seconds );

	// UGameEngine interface.
	virtual UBOOL Browse( FURL URL, const TMap<FString,FString>* TravelInfo, FString& Error );
	virtual ULevel* LoadMap( const FURL& URL, UPendingLevel* Pending, const TMap<FString,FString>* TravelInfo, FString& Error );
	virtual void SaveGame( INT Position );
	virtual void CancelPending();
	virtual void PaintProgress();
	virtual void UpdateConnectingMessage();
	virtual void BuildServerMasterMap( UNetDriver* NetDriver, ULevel* InLevel );
	virtual void NotifyLevelChange();
	void FixUpLevel();*/
};

#endif


#endif

#ifndef NAMES_ONLY
#undef AUTOGENERATE_NAME
#undef AUTOGENERATE_FUNCTION
#endif NAMES_ONLY

#if _MSC_VER
#pragma pack (pop)
#endif

/*-----------------------------------------------------------------------------
	The End.
-----------------------------------------------------------------------------*/
